/*
 * Copyright 1995, Jan A Knepper.
 *
 * Revision history :
 * ==================
 * 19950920 JAK   Created...
 */



#include <stdio.h>

#include "itbtrv32.hpp"



#ifdef __cplusplus
extern "C" {
#endif



#ifndef SizeOf                          // From DEFINE.H
#define SizeOf(s,f)    (sizeof(((s *)0)->f))
#endif



UT32PROC               btrcall32;
HGLOBAL                hglobal;
char                  *name      = "ITBTRV32";



BOOL WINAPI  DllEntryPoint ( HINSTANCE  instance, DWORD  reason, LPVOID /* reserved */ )
{
   switch ( reason )
   {
      case DLL_PROCESS_ATTACH :
         if ( UTRegister ( instance, "ITBTRV16.DLL", "BTRVINIT16", "BTRVCALL16", &btrcall32, NULL, NULL ) )
         {
            hglobal = GlobalAlloc ( GMEM_FIXED | GMEM_SHARE, sizeof ( BTRVPARM ) );
            if ( hglobal != NULL )
            {
               if ( GlobalLock ( hglobal ) == NULL )
               {
                  UTUnRegister ( instance );
                  GlobalFree ( hglobal );

                  return ( ReportError ( name, "GlobalLock" ) );
               }
            }
            else
            {
               UTUnRegister ( instance );

               return ( ReportError ( name, "GlobalAlloc" ) );
            }
         }
         else
            return ( ReportError ( name, "UTRegister" ) );

         return ( TRUE );

      case DLL_PROCESS_DETACH :
         if ( hglobal != NULL )
         {
            GlobalUnlock ( hglobal );
            GlobalFree   ( hglobal );
         }
         UTUnRegister ( instance );

         return ( TRUE );
   }

   return ( FALSE );
}



BTI_SINT PASCAL DLL_SPEC  BTRVINIT32 ( BTI_VOID )
{
   return (  0 );
}



BTI_SINT PASCAL DLL_SPEC  BTRVCALL32 ( BTI_WORD  operation, BTI_VOID_PTR  posBlock, BTI_VOID_PTR  dataBuffer, BTI_WORD_PTR  dataLength, BTI_VOID_PTR  keyBuffer, BTI_SINT  keyNumber )
{
   if ( ( dataLength != NULL ) && ( *dataLength > SizeOf ( BTRVPARM, dataBuffer ) ) )
      return ( B_DATALENGTH_ERROR );

   BTRVPARM   *ptr  = ( BTRVPARM * ) GlobalLock ( hglobal );
   BTI_SINT    stat;

   if ( ptr == NULL )
      return ( B_MEM_ALLOCATION_ERR );

   memset ( ptr, 0, sizeof ( BTRVPARM ) );

   ptr -> stat       =  0;
   ptr -> operation  =  operation;
   if ( posBlock != NULL )
      memcpy ( ptr -> posBlock  , posBlock  , SizeOf ( BTRVPARM, posBlock  ) );
   if ( ( dataBuffer != NULL ) && ( dataLength != NULL ) )
   {
      memcpy ( ptr -> dataBuffer, dataBuffer, *dataLength );
      ptr -> dataLength = *dataLength;
   }
   if ( keyBuffer != NULL )
      memcpy ( ptr -> keyBuffer , keyBuffer , SizeOf ( BTRVPARM, keyBuffer ) );
   ptr -> keyNumber  =  keyNumber;

   ( *btrcall32 ) ( ptr, 0L, NULL );

   stat = ptr -> stat;

   if ( ptr -> dataLength > SizeOf ( BTRVPARM, dataBuffer ) )
      return ( B_DATALENGTH_ERROR );

   if ( posBlock   != NULL )
      memcpy ( posBlock  , ptr -> posBlock  , SizeOf ( BTRVPARM, posBlock  ) );
   if ( dataLength != NULL )
      *dataLength = ptr -> dataLength;
   if ( dataBuffer != NULL )
      memcpy ( dataBuffer, ptr -> dataBuffer, *dataLength );
   if ( keyBuffer  != NULL )
      memcpy ( keyBuffer , ptr -> keyBuffer , SizeOf ( BTRVPARM, keyBuffer ) );
// keyNumber  = ptr -> keyNumber;

   GlobalUnlock ( hglobal );

   return ( stat );
}



BTI_SINT PASCAL DLL_SPEC  BTRVSTOP32 ( BTI_VOID )
{
   return (  0 );
}



DWORD PASCAL DLL_SPEC  ReportError ( const char  *title, const char  *subject )
{
   char   str [ 80 ];

   sprintf ( str, "%s Error %lu", subject, GetLastError () );
   MessageBox ( NULL, str, title, MB_OK );

   return ( FALSE );
}



#ifdef __cplusplus
}
#endif
