/*************************************************************************
**
**  Copyright 1982-1995 Btrieve Technologies, Inc. All Rights Reserved
**
*************************************************************************/
/***************************************************************************
   BTRSAMP.C
     This program demonstrates the C/C++ interface for Btrieve for DOS, 
     Extended DOS, OS2 (16 and 32-bit), MS Windows, NetWare NLM, MS
     Windows NT and Win32s.  It  creates a Btrieve file with 2 keys and
     then does some operations on that file.

     You can compile and run this program on any of the platforms
     supported by the interface modules.  Platforms are indicated by the
     platform switches listed in 'btrapi.h'.  For MS Windows you should
     make an application which allows standard output via printf().  Note
     that most C/C++ compilers support standard I/O Windows applications.
     For MS Windows NT or OS2, you should make a console application.

     See the prologue in 'btrapi.h' for information on how to select
     a target platform for your application.  You must specify a target
     platform.

****************************************************************************/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <btrapi.h>
#include <btrconst.h>

/***************************************************************************
   Constants
****************************************************************************/
   #define EXIT_WITH_ERROR    1
   #define TRUE               1
   #define FALSE              0
   #define VERSION_OFFSET     0
   #define REVISION_OFFSET    2
   #define PLATFORM_ID_OFFSET 4

/***************************************************************************
   Structure Type Definitions
****************************************************************************/
typedef struct
{
   BTI_SINT recLength;
   BTI_SINT pageSize;
   BTI_SINT indexCount;
   BTI_CHAR reserved[ 4 ];
   BTI_SINT flags;
   BTI_BYTE dupPointers;
   BTI_BYTE notUsed;
   BTI_SINT allocations;
} FILE_SPECS;

typedef struct
{
   BTI_SINT position;
   BTI_SINT length;
   BTI_SINT flags;
   BTI_CHAR reserved[ 4 ];
   BTI_CHAR type;
   BTI_CHAR null;
   BTI_CHAR notUsed[ 2 ];
   BTI_BYTE manualKeyNumber;
   BTI_BYTE acsNumber;
} KEY_SPECS;


/***************************************************************************
   Main
****************************************************************************/
int main( void )
{
   /* Btrieve Parameters */
   BTI_BYTE posBlock[ 128 ];
   BTI_BYTE dataBuf[ 255 ];
   BTI_WORD dataLen;
   BTI_BYTE keyBuf[ 255 ];
   BTI_WORD keyNum = 0;
   BTI_SINT status;

   unsigned char fileOpen = FALSE;
   unsigned char btrieveLoaded = FALSE;

   BTI_BYTE  clientID[ 16 ] = { 0,0,0,0,0,0,0,0,0,0,0,0,'M','T',0,1 };

   struct
   {
      FILE_SPECS fileSpecs;
      KEY_SPECS  keySpecs[ 2 ];
   } fileCreateBuf;

   /* DECLARE THE STRUCTURE FOR THE BTRIEVE RECORD. */
   #define FIELD_SIZE 21
   struct
   {
      char firstName[ FIELD_SIZE ];
      char middleName[ FIELD_SIZE ];
      char lastName[ FIELD_SIZE ];
   } nameRecord;

   /* PRINT THE PROGRAM TITLE */
   printf( "Btrieve C/C++ Interface Demo\n\n" );

   dataLen = sizeof( dataBuf );

   /************************************************************************
      BTRIEVE NOW SUPPORTS BTRVID ON ALL PLATFORMS.  WE WILL NOW EXERCISE
      BTRVID BY USING IT TO GET THE BTRIEVE VERSION. 
      REMEMBER: YOU CAN USE BTRVID TO PERFORM TRANSACTIONS OR RESETS ON A
      PER-CLIENT BASIS FROM THE SAME APPLICATION.
   *************************************************************************/
   status = BTRVID( B_VERSION, posBlock, &dataBuf, &dataLen, keyBuf,
               keyNum, clientID );
   if ( status == B_NO_ERROR )
   {
      printf( "Btrieve Version Is %d.%d %c\n",
         (*(BTI_WORD *)&dataBuf[ VERSION_OFFSET ]),
         (*(BTI_WORD *)&dataBuf[ REVISION_OFFSET ]),
         (*(BTI_CHAR *)&dataBuf[ PLATFORM_ID_OFFSET ] ));
      btrieveLoaded = TRUE;
   }
   else
   {
      printf( "Btrieve B_VERSION status = %d\n", status );
      if ( status != B_RECORD_MANAGER_INACTIVE )
         btrieveLoaded = TRUE;
   }

   /* RELEASE RESOURCES FOR clientID IN NON-DOS ENVIRONMENTS. */
   #if !defined( BTI_DOS ) && !defined( BTI_DOS_32R ) && \
   !defined( BTI_DOS_32B ) && !defined( BTI_DOS_32P )
   status = BTRVID( B_STOP, posBlock, &dataBuf, &dataLen, keyBuf,
               keyNum, clientID );
   printf( "Btrieve B_STOP status = %d\n", status );
   #endif

   /* CLEAR BUFFERS */
   if ( status == B_NO_ERROR )
   {
      memset( dataBuf, 0, sizeof( dataBuf ) );
      memset( keyBuf, 0, sizeof( keyBuf ) );
      memset( &fileCreateBuf, 0, sizeof( fileCreateBuf ) );

      /* CREATE THE BTRIEVE FILE 'XFACE.BTR' */
      fileCreateBuf.fileSpecs.recLength = sizeof( nameRecord );
      fileCreateBuf.fileSpecs.pageSize = 512;
      fileCreateBuf.fileSpecs.indexCount = 2;
      fileCreateBuf.fileSpecs.flags = 0;
      fileCreateBuf.fileSpecs.allocations = 0;

      fileCreateBuf.keySpecs[ 0 ].position = 1;                  /* index 1 */
      fileCreateBuf.keySpecs[ 0 ].length = FIELD_SIZE;
      fileCreateBuf.keySpecs[ 0 ].flags = EXTTYPE_KEY;
      fileCreateBuf.keySpecs[ 0 ].type = ZSTRING_TYPE;
      fileCreateBuf.keySpecs[ 0 ].null = 0;

      fileCreateBuf.keySpecs[1].position = 1+FIELD_SIZE;         /* index 2 */
      fileCreateBuf.keySpecs[1].length = FIELD_SIZE;
      fileCreateBuf.keySpecs[1].flags = EXTTYPE_KEY;
      fileCreateBuf.keySpecs[1].type = ZSTRING_TYPE;
      fileCreateBuf.keySpecs[1].null = 0;

      #ifdef BTI_NLM
      strcpy( (BTI_CHAR *)keyBuf, "sys:\\xface.btr" );
      #else
      strcpy( (BTI_CHAR *)keyBuf, "xface.btr" );
      #endif

      keyNum = 0;
      dataLen = sizeof( fileCreateBuf );
      status = BTRV( B_CREATE, posBlock, &fileCreateBuf, &dataLen, keyBuf,
                  keyNum );
      printf( "Btrieve B_CREATE status = %d\n", status );
   }

   /* OPEN XFACE.BTR */
   if ( status == B_NO_ERROR )
   {
      #ifdef BTI_NLM
      strcpy( (BTI_CHAR *)keyBuf, "sys:\\xface.btr" );
      #else
      strcpy( (BTI_CHAR *)keyBuf, "xface.btr" );
      #endif

      keyNum = 0;
      dataLen = 0;
      status = BTRV( B_OPEN, posBlock, dataBuf, &dataLen, keyBuf, keyNum );
      printf( "Btrieve B_OPEN status = %d\n", status );
      if ( status == B_NO_ERROR )
         fileOpen = TRUE;
   }

   /* INSERT A RECORD INTO XFACE.BTR */
   if ( status == B_NO_ERROR )
   {
      strcpy( nameRecord.firstName, "firstName" );
      strcpy( nameRecord.middleName, "middleName" );
      strcpy( nameRecord.lastName, "lastName" );
      dataLen = sizeof( nameRecord );
      keyNum = 0;
      status = BTRV( B_INSERT, posBlock, &nameRecord,
                  &dataLen, keyBuf, keyNum );
      printf( "Btrieve B_INSERT status = %d\n", status );
   }

   /* GET THE RECORD WITH STEP_FIRST */
   if ( status == B_NO_ERROR )
   {
      memset( &nameRecord, 0, sizeof(nameRecord) );
      dataLen = sizeof(nameRecord);
      status = BTRV( B_STEP_FIRST, posBlock, &nameRecord,
                  &dataLen, keyBuf, keyNum );
      printf( "Btrieve B_STEP_FIRST status = %d\n", status );
      if ( status == B_NO_ERROR )
      {
         printf( "Record is: %s %s %s\n", nameRecord.firstName,
            nameRecord.middleName, nameRecord.lastName );
      }
   }

   /* GET THE RECORD WITH GET_EQUAL */
   if ( status == B_NO_ERROR )
   {
      memset( &nameRecord, 0, sizeof(nameRecord) );
      dataLen = sizeof(nameRecord);
      strcpy( (BTI_CHAR *)keyBuf, "firstName" );
      status = BTRV( B_GET_EQUAL, posBlock, &nameRecord,
                  &dataLen, keyBuf, 0 );
      printf( "Btrieve B_GET_EQUAL status = %d\n", status );
      if ( status == B_NO_ERROR )
      {
         printf( "Record is: %s %s %s\n", nameRecord.firstName,
            nameRecord.middleName, nameRecord.lastName );
      }
   }

   /* ADD AN INDEX */
   if ( status == B_NO_ERROR )
   {
      memset( keyBuf, 0, sizeof(keyBuf) );
      fileCreateBuf.keySpecs[ 0 ].position = 1+FIELD_SIZE+FIELD_SIZE;
      fileCreateBuf.keySpecs[ 0 ].length = FIELD_SIZE;
      fileCreateBuf.keySpecs[ 0 ].flags = EXTTYPE_KEY;
      fileCreateBuf.keySpecs[ 0 ].type = ZSTRING_TYPE;
      fileCreateBuf.keySpecs[ 0 ].null = 0;
      dataLen = 16;
      status = BTRV( B_BUILD_INDEX, posBlock, &fileCreateBuf.keySpecs[0],
         &dataLen, keyBuf, keyNum );
      printf( "Btrieve B_BUILD_INDEX status = %d\n", status );
   }

   /* CLOSE THE FILE */
   if ( fileOpen )
   {
      dataLen = 0;
      status = BTRV( B_CLOSE, posBlock, dataBuf, &dataLen, keyBuf, keyNum );
      printf("Btrieve B_CLOSE status = %d\n", status );
   }

   /**********************************************************************
      ISSUE THE BTRIEVE STOP OPERATION - For multi-tasking environments,
      such as MS Windows, OS2, and NLM, 'stop' frees all Btrieve resources
      for this client.  In DOS and Extended DOS, it removes the Btrieve
      engine from memory, which we choose not to do in this example.  In
      multi-tasking environments, the engine will not unload on 'stop'
      unless it has no more clients.
   ***********************************************************************/
   #if !defined( BTI_DOS ) && !defined( BTI_DOS_32R ) && \
   !defined( BTI_DOS_32B ) && !defined( BTI_DOS_32P )
   if ( btrieveLoaded )
   {
      dataLen = 0;
      status = BTRV( B_STOP, posBlock, dataBuf, &dataLen, keyBuf, keyNum );
      printf( "Btrieve STOP status = %d\n", status );
      if ( status != B_NO_ERROR )
         status = EXIT_WITH_ERROR;
   }
   #endif

   return( (int)status );
}
